[CmdletBinding()]
param (
  [Parameter(Mandatory = $false)]
  [switch]
  $debugMode,
  [Parameter(Mandatory = $false)]
  [switch]
  $runElevated,
  [Parameter(Mandatory = $false)]
  [string]
  $server = "null",
  [Parameter(Mandatory = $false)]
  [string]
  $database = "null"
)

# - work through possible power bi desktop locations

$pbiDesktopFullName = 'C:\Program Files\Microsoft Power BI Desktop\bin\PBIDesktop.exe'
$pbiDesktopFullNameAlt = 'C:\Program Files (x86)\Microsoft Power BI Desktopbin\PBIDesktop.exe'
$winAppsFolder = "C:\Program Files\WindowsApps"

$pbiDesktop = $null

if (Test-Path $pbiDesktopFullName -PathType leaf) {

  $pbiDesktop = $pbiDesktopFullName

}
elseif (Test-Path $pbiDesktopFullNameAlt -PathType leaf) {

  $pbiDesktop = $pbiDesktopFullNameAlt

}
else {

  if ($runElevated -ne $true -and -not ([Security.Principal.WindowsPrincipal][Security.Principal.WindowsIdentity]::GetCurrent()).IsInRole([Security.Principal.WindowsBuiltInRole] "Administrator")) {

    $scriptFile = "'" + $myinvocation.mycommand.definition + "'"

    if ($debugMode -eq $true) {

      Start-Process -FilePath "powershell.exe" -Verb RunAs -WindowStyle Normal -ArgumentList @("-NoExit", "-NoProfile", "-ExecutionPolicy Bypass", "-Command & {&$scriptFile -debugMode -runElevated -server $server -database $database}")

    }
    else {

      Start-Process -FilePath "powershell.exe" -Verb RunAs -WindowStyle Hidden -ArgumentList @("-NoExit", "-NoProfile", "-ExecutionPolicy Bypass", "-Command & {&$scriptFile -runElevated -server $server -database $database}")

    }

  }

  $pbiDesktop = (Get-ChildItem -Path $winAppsFolder -File -Recurse -Filter "PBIDesktop.exe").FullName

}


# - identify where pbix file is

$pbixFileBasePath = 'C:\Program Files (x86)\Common Files\Microsoft Shared\Power BI Desktop\External Tools'
$pbixFileBasePathAlt = 'C:\Program Files\Common Files\Microsoft Shared\Power BI Desktop\External Tools'
$pbixFileCustSubPath = '\Practice Dataset\Practice Dataset.pbix'

$pbixFile = $null

if (Test-Path ($pbixFileBasePath + $pbixFileCustSubPath) -PathType leaf) {

  $pbixFile = '"' + $pbixFileBasePath + $pbixFileCustSubPath + '"'

}
elseif (Test-Path ($pbixFileBasePathAlt + $pbixFileCustSubPath) -PathType leaf) {

  $pbixFile = '"' + $pbixFileBasePathAlt + $pbixFileCustSubPath + '"'

}


# - start power bi desktop if arguments are valid

if ($pbiDesktop -And $pbixFile) {

  Start-Process -FilePath $pbiDesktop -ArgumentList @($pbixFile)
  
}


# - debug info

# $debugMode = $true

if ($debugMode -eq $true) {

  Write-Host "debugMode = $debugMode"
  Write-Host "runElevated = $runElevated"
  Write-Host "server = $server"
  Write-Host "database = $database"
  Write-Host ''

  Write-Host "pbixFile = $pbixFile"
  Write-Host ''

  Write-Host "pbiDesktop = $pbiDesktop"
  Write-Host ''

  Write-Host "scriptFile = $scriptFile"
  Write-Host ''

  Read-Host "Paused, press 'enter' to exit"

}